@@ -31,11 +31,12 @@ import os.path
 import re
 import io
 from ._base import TargetTemplate
-from chipwhisperer.hardware.naeusb.naeusb import NAEUSB,packuint32
-from chipwhisperer.hardware.naeusb.pll_cdce906 import PLLCDCE906
-from chipwhisperer.hardware.naeusb.fpga import FPGA
-from chipwhisperer.common.utils import util
-from chipwhisperer.common.utils.util import camel_case_deprecated, fw_ver_required
+from ...hardware.naeusb.naeusb import NAEUSB,packuint32
+from ...hardware.naeusb.pll_cdce906 import PLLCDCE906
+from ...hardware.naeusb.fpga import FPGA
+from ...common.utils import util
+from ...common.utils.util import camel_case_deprecated, fw_ver_required
+from ..scopes.cwhardware.ChipWhispererSAM3Update import SAMFWLoader
 
 from chipwhisperer.logging import *
 
@@ -106,6 +107,12 @@ class CW305(TargetTemplate):
     BATCHRUN_RANDOM_KEY = 0x2
     BATCHRUN_RANDOM_PT = 0x4
 
+    def upgrade_firmware(self):
+        """Attempt a firmware upgrade. See https://chipwhisperer.readthedocs.io/en/latest/firmware.html for more information.
+        """
+        prog = SAMFWLoader(self)
+        prog.auto_program()
+
     def __init__(self):
         import chipwhisperer as cw
         TargetTemplate.__init__(self)
@@ -1320,4 +1327,3 @@ class FPGAIO:
         self.spi1_set_cs_pin(True)
 
         return resp
-
