@@ -33,21 +33,25 @@ cw1200_v = [1, 30]
 cw1200_files = [("cw1200_firmware.zip", os.path.join(hwdir, r"capture/chipwhisperer-cw1200/cw1200_firmware.zip")),
                 ("CW1200_SAM3UFW.bin", os.path.join(hwdir, r"capture/chipwhisperer-cw1200/CW1200_SAM3UFW.bin"))]
 
-cwhusky_v = [0, 23]
-cwnano_v = [0, 30]
+cwhusky_v = [1, 1]
 cwhusky_files = [("husky_firmware.zip", os.path.join(hwdir, r"capture/chipwhisperer-husky/husky_firmware.zip")),
                 ("Husky.bin", os.path.join(hwdir, r"capture/chipwhisperer-husky/ChipWhisperer-Husky-SAM3U1C.bin"))]
 
+cwbergen_v = [0, 40]
+cwbergen_files = [("CW310.bin", os.path.join(hwdir, r"victims/cw310_bergen/CW310.bin"))]
+
+cwnano_v = [0, 30]
 #hardware\capture\chipwhisperer-nano\firmware\cwnano-firmware\Debug\cwnano-firmware.bin
 cwnano_files = [("SAM3U_CWNANO.bin", os.path.join(hwdir, r"capture/chipwhisperer-nano/firmware/cwnano-firmware/Debug/cwnano-firmware.bin"))]
 #List of files to generate
 file_list = [
-    #  ("cwnano.py" , cwnano_v , cwnano_files),
+      ("cwnano.py" , cwnano_v , cwnano_files),
     # ("cw305.py" , cw305_v , cw305_files),
 #    ("cwcr2.py" , cwcr2_v , cwcr2_files),
-    # ("cwlite.py", cwlite_v, cwlite_files),
+     ("cwlite.py", cwlite_v, cwlite_files),
     # ("cw1200.py", cw1200_v, cw1200_files),
-    ("cwhusky.py", cwhusky_v, cwhusky_files),
+    #("cwhusky.py", cwhusky_v, cwhusky_files),
+    #("cwbergen.py", cwbergen_v, cwbergen_files),
 ]
 
 for fdata in file_list:
